//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/8/21.
//

import Foundation

public struct GearAbsolute: RMCCommand {
    public var id = 25
    
    public var masterRegister: DMCPAddress
    public var masterRangeMin: Double
    public var masterRangeMax: Double
    public var slaveRangeMin: Double
    public var slaveRangeMax: Double
    public var endpointBehavior: EndpointBehavior
    
    public init(to register: DMCPAddress, masterFrom: Double, masterTo: Double, slaveFrom: Double, slaveTo: Double, atEndpoints: EndpointBehavior) {
        self.masterRegister = register
        self.masterRangeMin = masterFrom
        self.masterRangeMax = masterTo
        self.slaveRangeMin = slaveFrom
        self.slaveRangeMax = slaveTo
        self.endpointBehavior = atEndpoints
    }
    
    public var argArray: [Double] {[Double(self.masterRegister.intValue), self.masterRangeMin, self.masterRangeMax, self.slaveRangeMin, self.slaveRangeMax, self.endpointBehavior.rawValue]}
}

public struct GearPositionClutchByTime: RMCCommand {
    public var id = 30
    
    public var numerator: Double
    public var denominator: Double
    public var masterRegister: DMCPAddress
    public var clutchTime: Double
    
    public init(to register: DMCPAddress, withNumerator num: Double, withDenominator denom: Double, withClutchTime clutchTime: Double) {
        self.numerator = num
        self.denominator = denom
        self.masterRegister = register
        self.clutchTime = clutchTime
    }
    
    public var argArray: [Double] {[self.numerator, self.denominator, Double(self.masterRegister.intValue), self.clutchTime]}
}

public struct GearVelocityClutchByTime: RMCCommand {
    public var id = 31
    
    public var numerator: Double
    public var denominator: Double
    public var masterRegister: DMCPAddress
    public var clutchTime: Double
    
    public init(to register: DMCPAddress, withNumerator num: Double, withDenominator denom: Double, withClutchTime clutchTime: Double) {
        self.numerator = num
        self.denominator = denom
        self.masterRegister = register
        self.clutchTime = clutchTime
    }
    
    public var argArray: [Double] {[self.numerator, self.denominator, Double(self.masterRegister.intValue), self.clutchTime]}
}

public struct GearPositionClutchByDistance: RMCCommand {
    public var id = 32
    
    public var finalGearRatio: Double
    public var masterRegister: DMCPAddress
    public var masterSyncPos: Double
    public var slaveSyncPos: Double
    public var masterStartDistance: Double
    public var masterDirection: MovementDirection
    public var slaveDirection: MovementDirection
    
    public init(to register: DMCPAddress, withRatio ratio: Double, masterSyncPos: Double, slaveSyncPos: Double, masterStartDistance: Double, masterDirection: MovementDirection, slaveDirection: MovementDirection) {
        self.finalGearRatio = ratio
        self.masterRegister = register
        self.masterSyncPos = masterSyncPos
        self.slaveSyncPos = slaveSyncPos
        self.masterStartDistance = masterStartDistance
        self.masterDirection = masterDirection
        self.slaveDirection = slaveDirection
    }
    
    public var argArray: [Double] {[self.finalGearRatio, Double(self.masterRegister.intValue), self.masterSyncPos, self.slaveSyncPos, self.masterStartDistance, self.masterDirection.rawValue, self.slaveDirection.rawValue]}
}

public struct AdvancedGearMove: RMCCommand {
    public var id = 33
    
    public var slaveSyncPos: Double
    public var finalGearRatio: Double
    public var finalGearRatioRate: Double
    public var masterRegister: DMCPAddress
    public var masterSyncPosition: Double
    public var masterDirection: MovementDirection
    public var slaveDirection: MovementDirection
    
    public init(to register: DMCPAddress, withRatio ratio: Double, atRate rate: Double, masterSyncPos: Double, slaveSyncPos: Double, masterDirection: MovementDirection, slaveDirection: MovementDirection) {
        self.slaveSyncPos = slaveSyncPos
        self.finalGearRatio = ratio
        self.finalGearRatioRate = rate
        self.masterRegister = register
        self.masterSyncPosition = masterSyncPos
        self.masterDirection = masterDirection
        self.slaveDirection = slaveDirection
    }
    
    public var argArray: [Double] {[self.slaveSyncPos, self.finalGearRatio, self.finalGearRatioRate, Double(self.masterRegister.intValue), self.masterSyncPosition, self.masterDirection.rawValue, self.slaveDirection.rawValue]}
}

public struct Phasing: RMCCommand {
    public var id = 34
    
    public var masterPhaseOffset: Double
    public var masterDistance: Double
    
    public init(withMasterPhaseOffset offset: Double, withMasterDistance distance: Double) {
        self.masterPhaseOffset = offset
        self.masterDistance = distance
    }
    
    public var argArray: [Double] {[self.masterPhaseOffset, self.masterDistance]}
}

public struct GearedSlaveOffset: RMCCommand {
    public var id = 35
    
    public var slaveOffset: Double
    public var masterDistance: Double
    
    public init(withSlaveOffset offset: Double, withMasterDistance distance: Double) {
        self.slaveOffset = offset
        self.masterDistance = distance
    }
    
    public var argArray: [Double] {[self.slaveOffset, self.masterDistance]}
}

public struct GearPositionClutchByRate: RMCCommand {
    public var id = 39
    
    public var numerator: Double
    public var denominator: Double
    public var masterRegister: DMCPAddress
    public var accelRate: Double
    public var jerkRate: Double
    
    public init(to register: DMCPAddress, withNumerator num: Double, withDenominator denom: Double, withAccelRate accelRate: Double, withJerkRate jerkRate: Double) {
        self.numerator = num
        self.denominator = denom
        self.masterRegister = register
        self.accelRate = accelRate
        self.jerkRate = jerkRate
    }
    
    public var argArray: [Double] {[self.numerator, self.denominator, Double(self.masterRegister.intValue), self.accelRate, self.jerkRate]}
}

public struct TrackPosition: RMCCommand {
    public var id = 57
    
    public var masterRegister: DMCPAddress
    public var velocityLimit: Double
    public var accelLimit: Double
    public var jerkLimit: Double
    public var deadband: Double
    
    public init(of register: DMCPAddress, withVelocityLimit velLimit: Double, withAccelLimit accelLimit: Double, withJerkLimit jerkLimit: Double, deadband: Double) {
        self.masterRegister = register
        self.velocityLimit = velLimit
        self.accelLimit = accelLimit
        self.jerkLimit = jerkLimit
        self.deadband = deadband
    }
    
    public var argArray: [Double] {[Double(self.masterRegister.intValue), self.velocityLimit, self.accelLimit, self.jerkLimit, self.deadband]}
}

public struct TrackPositionIPD: RMCCommand {
    public var id = 58
    
    public var masterRegister: DMCPAddress
    public var velocityLimit: Double
    
    public init(of register: DMCPAddress, withVelocityLimit velLimit: Double) {
        self.masterRegister = register
        self.velocityLimit = velLimit
    }
    
    public var argArray: [Double] {[Double(self.masterRegister.intValue), self.velocityLimit]}
}
