//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/8/21.
//

import Foundation

public struct MoveAbsolute: RMCCommand {
    public var id = 20
    
    public var position: Double
    public var speed: Double
    public var accelRate: Double
    public var decelRate: Double
    public var inDirection: MovementDirection
    
    public init(to position: Double, atSpeed speed: Double, withAccelRate accelRate: Double, withDecelRate decelRate: Double, inDirection: MovementDirection = .nearest) {
        self.position = position
        self.speed = speed
        self.accelRate = accelRate
        self.decelRate = decelRate
        self.inDirection = inDirection
    }
    
    public var argArray: [Double] {
        [self.position, self.speed, self.accelRate, self.decelRate, self.inDirection.rawValue]
    }
}

public struct MoveRelative: RMCCommand {
    public var id = 21
    
    public var distance: Double
    public var speed: Double
    public var accelRate: Double
    public var decelRate: Double
    public var relativeTo: RelativeTo
    
    public init(by distance: Double, atSpeed speed: Double, withAccelRate accelRate: Double, withDecelRate decelRate: Double, relativeTo: RelativeTo) {
        self.distance = distance
        self.speed = speed
        self.accelRate = accelRate
        self.decelRate = decelRate
        self.relativeTo = relativeTo
    }
    
    public var argArray: [Double] {[self.distance, self.speed, self.accelRate, self.decelRate, self.relativeTo.rawValue]}
}

public struct QuickMoveAbsolute: RMCCommand {
    public var id = 15
    
    public var position: Double
    public var output: Double
    public var rampRate: Double
    public var decelRate: Double
    public var direction: MovementDirection
    
    public init(to position: Double, output: Double, withRampRate rampRate: Double, withDecelRate decelRate: Double, inDirection direction: MovementDirection) {
        self.position = position
        self.output = output
        self.rampRate = rampRate
        self.decelRate = decelRate
        self.direction = direction
    }
    
    public var argArray: [Double] {[self.position, self.output, self.rampRate, self.decelRate, self.direction.rawValue]}
}

public struct QuickMoveRelative: RMCCommand {
    public var id = 16
    
    public var distance: Double
    public var output: Double
    public var rampRate: Double
    public var decelRate: Double
    public var relativeTo: RelativeTo
    
    public init(by distance: Double, output: Double, withRampRate rampRate: Double, withDecelRate decelRate: Double, relativeTo: RelativeTo) {
        self.distance = distance
        self.output = output
        self.rampRate = rampRate
        self.decelRate = decelRate
        self.relativeTo = relativeTo
    }
    
    public var argArray: [Double] {[self.distance, self.output, self.rampRate, self.decelRate, self.relativeTo.rawValue]}
}

public struct TimeMoveAbsolute: RMCCommand {
    public var id = 23
    
    public var position: Double
    public var timeForMove: Double
    public var direction: MovementDirection
    
    public init(to position: Double, overSeconds timeForMove: Double, inDirection direction: MovementDirection) {
        self.position = position
        self.timeForMove = timeForMove
        self.direction = direction
    }
    
    public var argArray: [Double] {[self.position, self.timeForMove, self.direction.rawValue]}
}

public struct TimeMoveRelative: RMCCommand {
    public var id = 24
    
    public var distance: Double
    public var timeForMove: Double
    public var relativeTo: RelativeTo
    
    public init(by distance: Double, overSeconds timeForMove: Double, relativeTo: RelativeTo) {
        self.distance = distance
        self.timeForMove = timeForMove
        self.relativeTo = relativeTo
    }
    
    public var argArray: [Double] {[self.distance, self.timeForMove, self.relativeTo.rawValue]}
}

public struct AdvancedTimeMoveAbsolute: RMCCommand {
    public var id = 26
    
    public var position: Double
    public var finalVelocity: Double
    public var finalAccel: Double
    public var timeForMove: Double
    public var direction: MovementDirection
    
    public init(to position: Double, atVelocity velocity: Double, withAcceleration acceleration: Double, overSeconds timeForMove: Double, inDirection direction: MovementDirection) {
        self.position = position
        self.finalVelocity = velocity
        self.finalAccel = acceleration
        self.timeForMove = timeForMove
        self.direction = direction
    }
    
    public var argArray: [Double] {[self.position, self.finalVelocity, self.finalAccel, self.timeForMove, self.direction.rawValue]}
}

public struct AdvancedTimeMoveRelative: RMCCommand {
    public var id = 27
    
    public var distance: Double
    public var finalVelocity: Double
    public var finalAccel: Double
    public var timeForMove: Double
    public var relativeTo: RelativeTo
    
    public init(by distance: Double, atVelocity velocity: Double, withAcceleration acceleration: Double, overSeconds timeForMove: Double, relativeTo: RelativeTo) {
        self.distance = distance
        self.finalVelocity = velocity
        self.finalAccel = acceleration
        self.timeForMove = timeForMove
        self.relativeTo = relativeTo
    }
    
    public var argArray: [Double] {[self.distance, self.finalVelocity, self.finalAccel, self.timeForMove, self.relativeTo.rawValue]}
}

public struct MoveAbsoluteIPD: RMCCommand {
    public var id = 28
    
    public var position: Double
    public var maxSpeed: Double
    public var direction: MovementDirection
    
    public init(to position: Double, withMaxSpeed maxSpeed: Double, inDirection direction: MovementDirection) {
        self.position = position
        self.maxSpeed = maxSpeed
        self.direction = direction
    }
    
    public var argArray: [Double] {[self.position, self.maxSpeed, self.direction.rawValue]}
}

public struct MoveRelativeIPD: RMCCommand {
    public var id = 29
    
    public var distance: Double
    public var maxSpeed: Double
    public var relativeTo: RelativeTo
    
    public init(by distance: Double, withMaxSpeed maxSpeed: Double, relativeTo: RelativeTo) {
        self.distance = distance
        self.maxSpeed = maxSpeed
        self.relativeTo = relativeTo
    }
    
    public var argArray: [Double] {[self.distance, self.maxSpeed, self.relativeTo.rawValue]}
}
