//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/9/21.
//

import Foundation

public struct GearAbsolutePressureForce: RMCCommand {
    public var id = 59
    
    public var masterRegister: DMCPAddress
    public var masterRangeMin: Double
    public var masterRangeMax: Double
    public var slaveRangeMin: Double
    public var slaveRangeMax: Double
    public var endpointBehavior: EndpointBehavior
    
    public init(to register: DMCPAddress, masterFrom: Double, masterTo: Double, slaveFrom: Double, slaveTo: Double, atEndpoints: EndpointBehavior) {
        self.masterRegister = register
        self.masterRangeMin = masterFrom
        self.masterRangeMax = masterTo
        self.slaveRangeMin = slaveFrom
        self.slaveRangeMax = slaveTo
        self.endpointBehavior = atEndpoints
    }
    
    public var argArray: [Double] {[Double(self.masterRegister.intValue), self.masterRangeMin, self.masterRangeMax, self.slaveRangeMin, self.slaveRangeMax, self.endpointBehavior.rawValue]}
}

public struct TransitionDisablePressureForce: RMCCommand {
    public var id = 63
    public var argArray: [Double] {[]}
}

public struct TransitionRatePressureForce: RMCCommand {
    public var id = 64
    
    public var rate: Double
    public var accelRate: Double
    public var transitionType: TransitionType
    
    public init(withRate rate: Double, withAccelRate accelRate: Double, transitionType: TransitionType) {
        self.rate = rate
        self.accelRate = accelRate
        self.transitionType = transitionType
    }
    
    public var argArray: [Double] {[self.rate, self.accelRate, self.transitionType.rawValue]}
}

public struct SineStartPressureForce: RMCCommand {
    public var id = 76
    
    public var offset: Double
    public var amplitude: Double
    public var frequency: Double
    public var cycles: Double
    public var startLocation: SineStartLocation
    public var statusBlock: DMCPAddress?
    
    public init(withOffset offset: Double, amplitude: Double, frequency: Double, cycles: Double = 0, startLocation: SineStartLocation = .auto, statusBlock: DMCPAddress? = nil) {
        self.offset = offset
        self.amplitude = amplitude
        self.frequency = frequency
        self.cycles = cycles
        self.startLocation = startLocation
        self.statusBlock = statusBlock
    }
    
    public var argArray: [Double] {[self.offset, self.amplitude, self.frequency, self.cycles, self.startLocation.rawValue, self.statusBlock != nil ? Double(self.statusBlock!.intValue) : 0.0]}
}

public struct SineStopPressureForce: RMCCommand {
    public var id = 77
    
    public var stopLocation: SineStopLocation
    
    public init(at location: SineStopLocation) {
        self.stopLocation = location
    }
    
    public var argArray: [Double] {[self.stopLocation.rawValue]}
}
