//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/9/21.
//

import Foundation

public struct RampPressureForceRate: RMCCommand {
    public var id = 18
    
    public var pressureForce: Double
    public var rate: Double
    public var accelRate: Double
    
    public init(to pressureForce: Double, atRate rate: Double, withAccelRate accelRate: Double) {
        self.pressureForce = pressureForce
        self.rate = rate
        self.accelRate = accelRate
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rate, self.accelRate]}
}

public struct HoldCurrentPressureForce: RMCCommand {
    public var id = 19
    
    public var integratorPreload: Double
    
    public init(withIntegratorPreload integratorPreload: Double) {
        self.integratorPreload = integratorPreload
    }
    
    public var argArray: [Double] {[self.integratorPreload]}
}

public struct RampPressureForceSCurve: RMCCommand {
    public var id = 41
    
    public var pressureForce: Double
    public var rampTime: Double
    
    public init(to pressureForce: Double, overSeconds rampTime: Double) {
        self.pressureForce = pressureForce
        self.rampTime = rampTime
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rampTime]}
}

public struct RampPressureForceLinear: RMCCommand {
    public var id = 42
    
    public var pressureForce: Double
    public var rampTime: Double
    
    public init(to pressureForce: Double, overSeconds rampTime: Double) {
        self.pressureForce = pressureForce
        self.rampTime = rampTime
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rampTime]}
}

public struct StopPressureForce: RMCCommand {
    public var id = 43
    
    public var accelRate: Double
    
    public init(atAccelRate accelRate: Double) {
        self.accelRate = accelRate
    }
    
    public var argArray: [Double] {[self.accelRate]}
}

public struct EnterPressureForceControlAuto: RMCCommand {
    public var id = 44
    
    public var pressureForce: Double
    public var rampType: PressureForceRampType
    public var integratorPreload: Double
    
    public init(to pressureForce: Double, withRampType rampType: PressureForceRampType, withIntegratorPreload integratorPreload: Double) {
        self.pressureForce = pressureForce
        self.rampType = rampType
        self.integratorPreload = integratorPreload
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rampType.rawValue, self.integratorPreload]}
}

public struct EnterPressureForceControlTime: RMCCommand {
    public var id = 45
    
    public var pressureForce: Double
    public var rampType: PressureForceRampType
    public var rampTime: Double
    public var integratorPreload: Double
    
    public init(to pressureForce: Double, withRampType rampType: PressureForceRampType, overSeconds rampTime: Double, withIntegratorPreload integratorPreload: Double) {
        self.pressureForce = pressureForce
        self.rampType = rampType
        self.rampTime = rampTime
        self.integratorPreload = integratorPreload
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rampType.rawValue, self.rampTime, self.integratorPreload]}
}

public struct EnterPressureForceControlRate: RMCCommand {
    public var id = 46
    
    public var pressureForce: Double
    public var rate: Double
    public var accelRate: Double
    public var integratorPreload: Double
    
    public init(to pressureForce: Double, atRate rate: Double, atAccelRate accelRate: Double, withIntegratorPreload integratorPreload: Double) {
        self.pressureForce = pressureForce
        self.rate = rate
        self.accelRate = accelRate
        self.integratorPreload = integratorPreload
    }
    
    public var argArray: [Double] {[self.pressureForce, self.rate, self.accelRate, self.integratorPreload]}
}

public struct ChangeTargetParameterPressureForce: RMCCommand {
    public var id = 81
    
    public var parameter: CommandParameter
    public var newValue: Double
    public var rampType: RampType
    public var rampValue: Double
    
    public init(_ parameter: CommandParameter, toValue newValue: Double, withRampType rampType: RampType, toRampValue rampValue: Double) {
        self.parameter = parameter
        self.newValue = newValue
        self.rampType = rampType
        self.rampValue = rampValue
    }
    
    public var argArray: [Double] {[self.parameter.rawValue, self.newValue, self.rampType.rawValue, self.rampValue]}
}

public struct CurveStartPressureForce: RMCCommand {
    public var id = 87
    
    public var curveID: Int
    public var masterRegister: DMCPAddress
    public var cycles: Double
    
    public init(withID id: Int, on register: DMCPAddress, forCycles cycles: Double = 0.0) {
        self.curveID = id
        self.masterRegister = register
        self.cycles = cycles
    }
    
    public var argArray: [Double] {[Double(self.curveID), Double(self.masterRegister.intValue), self.cycles]}
}

public struct CurveStartAdvancedPressureForce: RMCCommand {
    public var id = 89
    
    public var curveID: Int
    public var masterRegister: DMCPAddress
    public var cycles: Double
    public var curveAlignment: CurveAlignment
    public var masterAlignment: MasterAlignment
    
    public var curveScale: Double
    public var curveOffset: Double
    public var masterScale: Double
    public var masterOffset: Double
    public var statusBlock: DMCPAddress?
    
    public init(withID id: Int, on register: DMCPAddress, forCycles cycles: Double, withCurveAlignment curveAlignment: CurveAlignment, withMasterAlignment masterAlignment: MasterAlignment, curveScale: Double = 1.0, curveOffset: Double = 0.0, masterScale: Double = 1.0, masterOffset: Double = 0.0, statusBlock: DMCPAddress? = nil) {
        self.curveID = id
        self.masterRegister = register
        self.cycles = cycles
        self.curveAlignment = curveAlignment
        self.masterAlignment = masterAlignment
        self.curveScale = curveScale
        self.curveOffset = curveOffset
        self.masterScale = masterScale
        self.masterOffset = masterOffset
        self.statusBlock = statusBlock
    }
    
    public var argArray: [Double] {
        var options: Double = 0.0
        options += self.curveAlignment.rawValue
        options += self.masterAlignment.rawValue
        switch self.masterAlignment {
        case .absolute(let alignment):
            options += alignment.rawValue
        case .relative(let alignment):
            options += alignment.rawValue
        }
        
        return [Double(self.curveID), Double(self.masterRegister.intValue), self.cycles, options, self.curveScale, self.curveOffset, self.masterScale, self.masterOffset, self.statusBlock != nil ? Double(self.statusBlock!.intValue) : 0.0]
    }
}
