//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/9/21.
//

import Foundation

public struct OffsetPosition: RMCCommand {
    public var id = 47
    
    public var change: Double
    
    public init(by change: Double) {
        self.change = change
    }
    
    public var argArray: [Double] {[self.change]}
}

public struct SetActualPosition: RMCCommand {
    public var id = 49
    
    public var position: Double
    
    public init(to position: Double) {
        self.position = position
    }
    
    public var argArray: [Double] {[self.position]}
}

public struct SetTargetPosition: RMCCommand {
    public var id = 48
    
    public var position: Double
    
    public init(to position: Double) {
        self.position = position
    }
    
    public var argArray: [Double] {[self.position]}
}

public struct SetActualPressureForce: RMCCommand {
    public var id = 65
    
    public var pressureForce: Double
    
    public init(to pressureForce: Double) {
        self.pressureForce = pressureForce
    }
    
    public var argArray: [Double] {[self.pressureForce]}
}

public struct SelectGainSet: RMCCommand {
    public var id = 75
    
    public var gainSet: GainSet
    
    public init(_ gainSet: GainSet) {
        self.gainSet = gainSet
    }
    
    public var argArray: [Double] {[self.gainSet.rawValue]}
}

public struct SetPosVelCtrlMode: RMCCommand {
    public var id = 68
    
    public var controlMode: ControlMode
    
    public init(_ controlMode: ControlMode) {
        self.controlMode = controlMode
    }
    
    public var argArray: [Double] {[self.controlMode.rawValue]}
}

public struct FeedForwardAdjust: RMCCommand {
    public var id = 69
    public var argArray: [Double] {[]}
}

public struct IntegratorAdjust: RMCCommand {
    public var id = 70
    
    public var integralOutput: Double
    public var integratorSelect: IntegratorType
    
    public init(outputPercent output: Double, withIntegrator integrator: IntegratorType) {
        self.integralOutput = output
        self.integratorSelect = integrator
    }
    
    public var argArray: [Double] {[self.integralOutput, self.integratorSelect.rawValue]}
}

public struct SetIntegratorMode: RMCCommand {
    public var id = 71
    
    public var mode: IntegratorMode
    
    public init(_ mode: IntegratorMode) {
        self.mode = mode
    }
    
    public var argArray: [Double] {[self.mode.rawValue]}
}

public struct SetControlDirection: RMCCommand {
    public var id = 96
    
    public var direction: ControlDirection
    
    public init(_ direction: ControlDirection) {
        self.direction = direction
    }
    
    public var argArray: [Double] {[self.direction.rawValue]}
}

public struct ReadRegister: RMCCommand {
    public var id = 111
    
    public var address: DMCPAddress
    
    public init(_ address: DMCPAddress) {
        self.address = address
    }
    
    public var argArray: [Double] {[Double(self.address.intValue)]}
}

public struct WriteRegister: RMCCommand {
    public var id = 112
    
    public var address: DMCPAddress
    public var value: Double
    
    public init(_ address: DMCPAddress, withValue value: Double) {
        self.address = address
        self.value = value
    }
    
    public var argArray: [Double] {[Double(self.address.intValue), self.value]}
}
