//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/9/21.
//

import Foundation

public struct ArmHome: RMCCommand {
    public var id = 50
    
    public var homePosition: Double
    public var triggerType: TriggerType
    public var repeatMode: RepeatMode
    public var homeInput: Int
    
    public init(homePosition: Double, triggerType: TriggerType, repeatMode: RepeatMode, homeInput: Int) {
        self.homePosition = homePosition
        self.triggerType = triggerType
        self.repeatMode = repeatMode
        self.homeInput = homeInput
    }
    
    public var argArray: [Double] {[self.homePosition, self.triggerType.rawValue, self.repeatMode.rawValue, Double(self.homeInput)]}
}

public struct DisarmHome: RMCCommand {
    public var id = 51
    public var argArray: [Double] {[]}
}

public struct ArmRegistration: RMCCommand {
    public var id = 52
    
    public var registrationNumber: Int
    public var registrationInput: Int
    public var inputEdge: InputEdge
    
    public init(_ registrationNumber: Int, usingInput registrationInput: Int, onEdge inputEdge: InputEdge) {
        self.registrationNumber = registrationNumber
        self.registrationInput = registrationInput
        self.inputEdge = inputEdge
    }
    
    public var argArray: [Double] {[Double(self.registrationNumber), Double(self.registrationInput), self.inputEdge.rawValue]}
}

public struct DisarmRegistration: RMCCommand {
    public var id = 53
    
    public var registrationNumber: Int
    
    public init(_ registrationNumber: Int) {
        self.registrationNumber = registrationNumber
    }
    
    public var argArray: [Double] {[Double(self.registrationNumber)]}
}

public struct LearnZAlignment: RMCCommand {
    public var id = 54
    public var argArray: [Double] {[]}
}

public struct PauseResumeLog: RMCCommand {
    public var id = 95
    
    public var logCommand: PauseResume
    
    public init(_ logCommand: PauseResume) {
        self.logCommand = logCommand
    }
    
    public var argArray: [Double] {[self.logCommand.rawValue]}
}

public struct ArmEventTimer: RMCCommand {
    public var id = 105
    
    public var moduleSlot: Int
    public var eventTimer: Int
    public var eventSource: EventSource
    public var timerMode: TimerMode
    
    public init(moduleSlot: Int, eventTimer: Int, eventSource: EventSource, timerMode: TimerMode) {
        self.moduleSlot = moduleSlot
        self.eventTimer = eventTimer
        self.eventSource = eventSource
        self.timerMode = timerMode
    }
    
    public var argArray: [Double] {[Double(self.moduleSlot), Double(self.eventTimer), self.eventSource.rawValue, self.timerMode.rawValue]}
}

public struct DisarmEventTimer: RMCCommand {
    public var id = 106
    
    public var moduleSlot: Int
    public var eventTimer: Int
    
    public init(moduleSlot: Int, eventTimer: Int) {
        self.moduleSlot = moduleSlot
        self.eventTimer = eventTimer
    }
    
    public var argArray: [Double] {[Double(self.moduleSlot), Double(self.eventTimer)]}
}

public struct UpdateFlash: RMCCommand {
    public var id = 110
    
    public var sectionToStore: Int
    
    public init(_ sectionToStore: Int = 0) {
        self.sectionToStore = sectionToStore
    }
    
    public var argArray: [Double] {[Double(self.sectionToStore)]}
}

public struct SaveControllerImage: RMCCommand {
    public var id = 120
    public var argArray: [Double] {[]}
}

public struct RestoreControllerImage: RMCCommand {
    public var id = 121
    public var argArray: [Double] {[]}
}
