//
//  DMCPDataTypes.swift
//  
//
//  Created by Malcolm Anderson on 6/7/21.
//

import Foundation


public enum DMCPData {
    // This allows us to store both integers and reals (aka floats)
    // Concept from here: https://stackoverflow.com/a/49008453
    case int(Int32?)
    case real(Float32?)
    
    public enum DataType {
        case int
        case real
    }
}

public struct DMCPAddress: CustomStringConvertible, Hashable {
    public var description: String { "%MD\(self.file).\(self.element)" }
    public var isEmpty: Bool { self.file == 0 && self.element == 0 }
    
    public var file: UInt16 = 56
    public var element: UInt16 = 0
    
    public init(_ file: UInt16, _ element: UInt16) {
        self.file = file
        self.element = element
    }
    
    public init(fromWord word: Int) {
        self.file = UInt16((word >> 12) & 0xFFF)
        self.element = UInt16(word & 0xFFF)
    }
    
    public var intValue: Int { Int(self.file * 4096 + self.element) }
}
