//
//  DMCPResponseCode.swift
//  
//
//  Created by Malcolm Anderson on 6/4/21.
//

import Foundation

internal enum DMCPResponseCode: UInt8 {
    case success = 0
    case malformed = 1
    case tooLong = 2
    case invalidAddress = 3
    
    case unknownResponseCode
}

/// Error response codes from the RMC.
public enum DMCPResponseError: Error {
    /// The request was not in the correct format
    case malformed
    
    /// The request was too long
    case tooLong
    
    /// The request involved an invalid address
    case invalidAddress
    
    
    static func fromCode(_ code: DMCPResponseCode) -> DMCPResponseError? {
        switch code {
        case .malformed: return .malformed
        case .tooLong: return .tooLong
        case .invalidAddress: return .invalidAddress
        default: return nil
        }
    }
}

/// Other errors
public enum DMCPPayloadError: Error {
    /// The response was not in the correct format and couldn't be parsed
    case badResponse
}
