//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/21/21.
//

import Foundation

class PlotUtilities {
    static func packageData(_ data: [DMCPAddress: [Double?]], forPlot id: Int, connection: DMCPConnection, _ onResult: @escaping (_ error: Error?, _ data: RMCPlotData?) -> Void) {
        // Creating a dictionary that maps the address strings to non-optional Double values
        let keys = Array(data.keys)
        var samplesUnwrapped: [String: [Double]] = [:]
        
        for address in keys {
            var newDoubleArray: [Double] = []
            
            for item in data[address]! {
                newDoubleArray.append(item!)
            }
            
            samplesUnwrapped[String(describing: address)] = newDoubleArray
        }
        
        // Next, let's get the name that each quantity corresponds to
        
        var currentQuantityIndex = 0
        var quantities: [RMCPlotQuantity] = []
        
        func addQuantity() {
            let currentQuantityAddress = keys[currentQuantityIndex]
            
            connection.getAddressLabel(for: currentQuantityAddress) { error, name in
                if error != nil {
                    print("Error while trying to get address name: \(error!)")
                }
                
                let newQuantity = RMCPlotQuantity(
                    name: name!.name,
                    unit: name!.unit.rawValue,
                    scaleGroup: name!.unit.rawValue,
                    centerOnZero: name!.unit.centerOnZero,
                    formula: String(describing: currentQuantityAddress),
                    color: RMCPlotColor(seed: currentQuantityIndex)
                )
                
                quantities.append(newQuantity)
                
                currentQuantityIndex += 1
                
                if currentQuantityIndex < data.count {
                    addQuantity()
                } else {
                    // All done getting quantity info!
                    // Now we just need to get the sample period
                    
                    connection.getPlotSamplePeriod(forPlot: id) { error, samplePeriod in
                        if error != nil {
                            print("Error while trying to get sample period: \(error!)")
                        }
                        
                        // Create the plot data object
                        let plotDataObject = RMCPlotData(samplePeriod: samplePeriod!, quantities: quantities, samples: samplesUnwrapped)
                        
                        onResult(nil, plotDataObject)
                    }
                }
            }
        }
        
        addQuantity()
    }
}
