//
//  RMCCommands.swift
//  
//
//  Created by Malcolm Anderson on 6/8/21.
//

import Foundation

public protocol RMCCommand {
    var id: Int { get }
    var argArray: [Double] { get }
}

extension RMCCommand {
    func array() -> [Double] {
        return [Double(self.id)] + self.argArray
    }
}

public enum MovementDirection: Double {
    case negative = -1.0
    case nearest = 0.0
    case positive = 1.0
    case current = 2.0
    case absolute = 3.0
}

public enum RelativeTo: Double {
    case commandPosition = 0.0
    case actualPosition = 1.0
    case targetPosition = 2.0
}

public enum EndpointBehavior: Double {
    case fault = 0.0
    case truncate = 1.0
    case extrapolate = 2.0
}

public enum SineStartLocation: Double {
    case auto = 0.0
    case midPos = 1.0
    case posPeak = 2.0
    case midNeg = 3.0
    case negPeak = 4.0
}

public enum SineStopLocation: Double {
    case nextCycle = 0.0
    case middle = 1.0
    case posPeak = 2.0
    case negPeak = 3.0
    case nextPeak = 4.0
}

public enum CommandParameter: Double {
    case offset = 0.0
    case amplitude = 1.0
    case frequency = 2.0
    case cycles = 3.0
    case phase = 4.0
}

public enum RampType: Double {
    case time = 0.0
    case cycles = 1.0
    case rate = 2.0
    case timeLog = 3.0
    case cyclesLog = 4.0
    case time5th = 5.0
}

public enum CurveInterpolationMethod: Double {
    case constant = 0.0
    case linear = 1.0
    case cubic = 2.0
}

public enum CurveLifeCycle: Double {
    case standard = 0.0
    case startOnce = 1.0
    case completeOnce = 2.0
    case permanent = 3.0
}

public enum CurveAlignment: Double {
    case absolute = 0.0
    case relative = 1.0
}

public enum MasterAlignment {
    case relative(RelMasterAlignmentEndpointBehavior)
    case absolute(AbsMasterAlignmentEndpointBehavior)
    
    public var rawValue: Double {
        switch self {
        case .relative(_):
            return 0.0
        case .absolute(_):
            return 2.0
        }
    }
}

public enum AbsMasterAlignmentEndpointBehavior: Double {
    case fault = 0.0
    case truncate = 4.0
    case extrapolate = 8.0
}

public enum RelMasterAlignmentEndpointBehavior: Double {
    case standard = 0.0
    case truncate = 4.0
    case truncateAndEnd = 8.0
}

public enum VelocityDirection: Double {
    case positive = 1.0
    case negative = -1.0
    case current = 2.0
    case signed = 4.0
}

public enum TransitionType: Double {
    case seek = 0.0
    case reach = 1.0
    case superimposed = 2.0
}

public enum GainSet: Double {
    case zero = 0.0
    case one = 1.0
}

public enum ControlMode: Double {
    case positionPID = 0.0
    case positionIPD = 1.0
    case velocityPID = 4.0
    case velocityIPD = 5.0
}

public enum IntegratorType: Double {
    case active = 0.0
    case posVel = 1.0
    case prsFrc = 2.0
}

public enum IntegratorMode: Double {
    case held = 0.0
    case active = 1.0
    case zero = 2.0
    case tgDone = 3.0
    case decel = 4.0
}

public enum ControlDirection: Double {
    case positive = 1.0
    case negative = -1.0
}

public enum PressureForceRampType: Double {
    case linear = 0.0
    case sCurve = 1.0
}

public enum PressureForceLimitMode: Double {
    case disabled = 0.0
    case positive = 1.0
    case negative = 2.0
    case bidirectional = 3.0
}

public enum TriggerType: Double {
    case hRising = 0.0
    case hFalling = 1.0
    case z = 2.0
    case zAndH = 3.0
    case zAndNotH = 4.0
    case absoluteAdjustHRising = 5.0
    case absoluteAdjustHFalling = 6.0
    case absoluteAdjustImmediate = 7.0
}

public enum RepeatMode: Double {
    case single = 0.0
    case `repeat` = 1.0
}

public enum InputEdge: Double {
    case falling = 0.0
    case rising = 1.0
}

public enum PauseResume: Double {
    case pause = 0.0
    case resume = 1.0
}

public enum EventSource: Double {
    case quadCount = 0.0
    case in0RisingEdge = 1.0
    case in0FallingEdge = 2.0
    case in1RisingEdge = 3.0
    case in1FallingEdge = 4.0
}

public enum TimerMode: Double {
    case oneShot = 0.0
    case continuous = 1.0
    case alternating = 2.0
}
