//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/16/21.
//

import Foundation

public class RMC150: RMC {
    public static let displayName: String = "RMC150"
    public static let numberOfAxes = 16
    
    public static let serialNumberAddr: DMCPAddress? = DMCPAddress(7,14)
    
    public static let currentAxisDefinitionsAddr: DMCPAddress? = DMCPAddress(43,0)
    
    public static let numberOfAllocatedTasksAddr: DMCPAddress? = DMCPAddress(44,4)
    public static let controllerNameAddr: DMCPAddress? = DMCPAddress(44,16)
    
    public static let firstAxisNameAddr: DMCPAddress? = DMCPAddress(49,0)
    public static let firstAxisCommandAddr: DMCPAddress? = DMCPAddress(40,0)
    
    public static let currentPlotLayoutAddr: DMCPAddress? = DMCPAddress(95,0)
    public static let maximumPlotsAddr: DMCPAddress? = DMCPAddress(95,1)
    public static let maximumPlotsDataItemsAddr: DMCPAddress? = DMCPAddress(95,2)
    
    
    public static func axisNameAddr(_ axis: Int) -> DMCPAddress? {
        var addr = firstAxisNameAddr
        addr!.element += UInt16(axis * 16)
        return addr
    }
    
    public static func axisCommandAddr(_ axis: Int) -> DMCPAddress? {
        var addr = firstAxisCommandAddr
        addr!.element += UInt16(axis * 10)
        return addr
    }
    
    private static func offsetByArg(_ address: DMCPAddress, fileBy amount: Int) -> DMCPAddress? {
        var addr = address
        addr.file += UInt16(amount)
        return addr
    }
    
    public static var firstPlotSamplesAddr: DMCPAddress? = DMCPAddress(96,1)
    public static func plotSamplesAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotSamplesAddr!, fileBy: plot)
    }
    
    public static let firstPlotSamplePeriodAddr: DMCPAddress? = DMCPAddress(96,2)
    public static func plotSamplePeriodAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotSamplePeriodAddr!, fileBy: plot)
    }
    
    public static var firstPlotAxisOwnerAddr: DMCPAddress? = DMCPAddress(96,3)
    public static func plotAxisOwnerAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotAxisOwnerAddr!, fileBy: plot)
    }
    
    public static var firstPlotTriggerPositionAddr: DMCPAddress? = DMCPAddress(96,4)
    public static func plotTriggerPositionAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotTriggerPositionAddr!, fileBy: plot)
    }
    
    public static var firstPlotTriggerTypeAddr: DMCPAddress? = DMCPAddress(96,5)
    public static func plotTriggerTypeAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotTriggerTypeAddr!, fileBy: plot)
    }
    
    public static var firstPlotIDAddr: DMCPAddress? = DMCPAddress(96,8)
    public static func plotIDAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotIDAddr!, fileBy: plot)
    }
    
    public static var firstPlotStateAddr: DMCPAddress? = DMCPAddress(96,9)
    public static func plotStateAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotStateAddr!, fileBy: plot)
    }
    
    public static var firstPlotCapturedSamplesAddr: DMCPAddress? = DMCPAddress(96,10)
    public static func plotCapturedSamplesAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotCapturedSamplesAddr!, fileBy: plot)
    }
    
    public static var firstPlotTriggerTimeAddr: DMCPAddress? = DMCPAddress(96,12)
    public static func plotTriggerTimeAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotTriggerTimeAddr!, fileBy: plot)
    }
    
    public static var firstPlotTriggerIndexAddr: DMCPAddress? = DMCPAddress(96,13)
    public static func plotTriggerIndexAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotTriggerIndexAddr!, fileBy: plot)
    }
    
    public static var firstPlotDataSetsAddressesAddr: DMCPAddress? = DMCPAddress(96,16)
    public static func plotDataSetsAddressesAddr(_ plot: Int) -> DMCPAddress? {
        return offsetByArg(firstPlotDataSetsAddressesAddr!, fileBy: plot)
    }
    
    public static var staticPlotReadMaxPlots = 8
    public static var staticPlotReadMaxDataSets = 4
    public static var firstPlotStaticUploadAreaAddr: DMCPAddress? = DMCPAddress(112,0)
    public static func plotStaticUploadAreaAddr(forPlot plot: Int, dataSet: Int) -> DMCPAddress? {
        var address = firstPlotStaticUploadAreaAddr
        address?.file += UInt16(4 * plot + dataSet)
        return address
    }
    
    
    // NOTE: Could be wrong
    public static var firstPlotDynamicUploadModeStatusAddr: DMCPAddress? = DMCPAddress(104,0)
    public static func plotDynamicUploadModeStatusAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicUploadModeStatusAddr!, fileBy: plot)
    }
    
    public static var firstPlotDynamicRequestedReadSamplesAddr: DMCPAddress? = DMCPAddress(104,1)
    public static func plotDynamicRequestedReadSamplesAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicRequestedReadSamplesAddr!, fileBy: plot)
    }
    
    public static var firstPlotDynamicCurrentIndexAddr: DMCPAddress? = DMCPAddress(104,2)
    public static func plotDynamicCurrentIndexAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicCurrentIndexAddr!, fileBy: plot)
    }
    
    public static var firstPlotDynamicIDAddr: DMCPAddress? = DMCPAddress(104,3)
    public static func plotDynamicIDAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicIDAddr!, fileBy: plot)
    }
    
    public static var firstPlotDynamicSamplesUploadedAddr: DMCPAddress? = DMCPAddress(104,4)
    public static func plotDynamicSamplesUploadedAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicSamplesUploadedAddr!, fileBy: plot)
    }
    
    public static var firstPlotDynamicDataAddr: DMCPAddress? = DMCPAddress(104,5)
    public static func plotDynamicDataAddr(forPlot plot: Int) -> DMCPAddress? {
        offsetByArg(firstPlotDynamicDataAddr!, fileBy: plot)
    }
}
