//
//  File.swift
//  
//
//  Created by Malcolm Anderson on 6/16/21.
//

import Foundation

public protocol RMC {
    /// The name of the RMC model.
    static var displayName: String { get }
    
    /// The number of axes the RMC has.
    static var numberOfAxes: Int { get }
    
    
    static var serialNumberAddr: DMCPAddress? { get }
    
    // - MARK: Axis definitions
    static var currentAxisDefinitionsAddr: DMCPAddress? { get }
    
    // - MARK: Controller status/parameters
    static var controllerNameAddr: DMCPAddress? { get }
    static var numberOfAllocatedTasksAddr: DMCPAddress? { get }
    
    // - MARK: Generic axis name
    static var firstAxisNameAddr: DMCPAddress? { get }
    static func axisNameAddr(_ axis: Int) -> DMCPAddress?
    
    // - MARK: Generic axis commands
    static var firstAxisCommandAddr: DMCPAddress? { get }
    static func axisCommandAddr(_ axis: Int) -> DMCPAddress?
    
    // - MARK: Plot configuration
    static var currentPlotLayoutAddr: DMCPAddress? { get }
    static var maximumPlotsAddr: DMCPAddress? { get }
    static var maximumPlotsDataItemsAddr: DMCPAddress? { get }
    
    // - MARK: Individual Plot Properties
    static var firstPlotSamplesAddr: DMCPAddress? { get }
    static func plotSamplesAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotSamplePeriodAddr: DMCPAddress? { get }
    static func plotSamplePeriodAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotAxisOwnerAddr: DMCPAddress? { get }
    static func plotAxisOwnerAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotTriggerPositionAddr: DMCPAddress? { get }
    static func plotTriggerPositionAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotTriggerTypeAddr: DMCPAddress? { get }
    static func plotTriggerTypeAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotIDAddr: DMCPAddress? { get }
    static func plotIDAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotStateAddr: DMCPAddress? { get }
    static func plotStateAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotCapturedSamplesAddr: DMCPAddress? { get }
    static func plotCapturedSamplesAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotTriggerTimeAddr: DMCPAddress? { get }
    static func plotTriggerTimeAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotTriggerIndexAddr: DMCPAddress? { get }
    static func plotTriggerIndexAddr(_ plot: Int) -> DMCPAddress?
    
    static var firstPlotDataSetsAddressesAddr: DMCPAddress? { get }
    static func plotDataSetsAddressesAddr(_ plot: Int) -> DMCPAddress?
    
    static var staticPlotReadMaxPlots: Int { get }
    static var staticPlotReadMaxDataSets: Int { get }
    static var firstPlotStaticUploadAreaAddr: DMCPAddress? { get }
    static func plotStaticUploadAreaAddr(forPlot plot: Int, dataSet: Int) -> DMCPAddress?
    
    
    static var firstPlotDynamicUploadModeStatusAddr: DMCPAddress? { get }
    static var firstPlotDynamicRequestedReadSamplesAddr: DMCPAddress? { get }
    static var firstPlotDynamicCurrentIndexAddr: DMCPAddress? { get }
    static var firstPlotDynamicIDAddr: DMCPAddress? { get }
    static var firstPlotDynamicSamplesUploadedAddr: DMCPAddress? { get }
    static var firstPlotDynamicDataAddr: DMCPAddress? { get }
    
    static func plotDynamicUploadModeStatusAddr(forPlot plot: Int) -> DMCPAddress?
    static func plotDynamicRequestedReadSamplesAddr(forPlot plot: Int) -> DMCPAddress?
    static func plotDynamicCurrentIndexAddr(forPlot plot: Int) -> DMCPAddress?
    static func plotDynamicIDAddr(forPlot plot: Int) -> DMCPAddress?
    static func plotDynamicSamplesUploadedAddr(forPlot plot: Int) -> DMCPAddress?
    static func plotDynamicDataAddr(forPlot plot: Int) -> DMCPAddress?
}

public enum RMCError: Error {
    case unknownRMC
}
