import XCTest
@testable import SwiftDMCP

final class PlotTests: XCTestCase {
    let rmcAddress = "192.168.6.163"
    
    func testBasicPlot() {
        let c = DMCPConnection(address: rmcAddress)
        var e = self.expectation(description: "Connect to RMC")
        
        c.connect { error in
            XCTAssertNil(error)
            e.fulfill()
        }
        
        waitForExpectations(timeout: 5, handler: nil)
        
        e = self.expectation(description: "Receive response from RMC")
        
        let plotReader = BasicPlotReader(c)
        
        plotReader.readPlot(0) { error, data in
            XCTAssertNil(error)
            
            self.makeJSON(data!)
            e.fulfill()
        }
        
        waitForExpectations(timeout: 5, handler: nil)
    }
    
    func makeJSON(_ data: RMCPlotData) {
        let encoder = JSONEncoder()
        
        let jsonData = try! encoder.encode(data)
        let jsonString = String(data: jsonData, encoding: .utf8)!
        
        let path = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask)[0].appendingPathComponent("plotout.json")
        print("path is \(path)")
        try! jsonString.write(to: path, atomically: true, encoding: .utf8)
    }
    
    func makeCSV(_ data: [DMCPAddress: [Double?]]) -> String {
        var output = ""
        
        let stringKeys: [String] = data.keys.map { String(describing: $0) }
        output += stringKeys.joined(separator: ",")
        
        output += "\n"
        
        // Start putting in the data
        for i in 0..<1000 {
            var row = ""
            for j in 0..<4 {
                let currentValue = Array(data.values)[j][i]
                row += "\(currentValue!)\(j < 3 ? "," : "")"
            }
            output += "\(row)\n"
        }
        
        return output
    }

    func testColors() {
        for i in 0...10 {
            _ = RMCPlotColor(seed: i)
        }
    }
}
