    import XCTest
    @testable import SwiftDMCP

    final class SwiftDMCPTests: XCTestCase {
        let rmcAddress = "192.168.6.163"
        let rmcPort = 1324
        
        func testConnect() {
            let connection = DMCPConnection(address: rmcAddress, port: rmcPort)
            
            connection.connect { error in
                XCTAssert(error == nil)
            }
            
            connection.disconnect()
        }
        
        func testBadConnect() {
            let connection = DMCPConnection(address: "haha", port: 52)
            
            connection.connect { error in
                XCTAssert(error != nil)
            }
            
            connection.disconnect()
        }
        
        func testWriteRequestInt() {
            let c = DMCPConnection(address: rmcAddress)
            
            c.connect { error in
                XCTAssert(error == nil)
            }
            
            let expectation = self.expectation(description: "Receive response from RMC")
            
            let req = DMCPWriteRequest(56, 0, values: [.int(0x11223344)]) { error in
                XCTAssert(error == nil)
                expectation.fulfill()
            }
            c.sendRequest(req)
            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testWriteRequestReal() {
            let c = DMCPConnection(address: rmcAddress)
            
            c.connect { error in
                XCTAssert(error == nil)
            }
            
            let expectation = self.expectation(description: "Receive response from RMC")
            
            let req = DMCPWriteRequest(56, 0, values: [.real(2.235)]) { error in
                XCTAssert(error == nil)
                expectation.fulfill()
            }
            c.sendRequest(req)
            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testWriteRequestMultiple() {
            let c = DMCPConnection(address: rmcAddress)

            c.connect { error in
                XCTAssert(error == nil)
            }

            let expectation = self.expectation(description: "Receive response from RMC")

            let req = DMCPWriteRequest(56, 0, values: [.real(0.4), .real(14.32), .int(3), .int(7)]) { error in
                XCTAssert(error == nil)
                expectation.fulfill()
            }
            c.sendRequest(req)

            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testReadRequest() {
            let c = DMCPConnection(address: rmcAddress)
            
            c.connect { error in XCTAssert(error == nil) }
            
            let expectation = self.expectation(description: "Receive response from RMC")
            
            let req = DMCPReadRequest(56, 0, readTypes: [.real, .real, .int, .int]) { error, values in
                XCTAssert(error == nil)
                print(values)
                expectation.fulfill()
                
                
            }
            
            c.sendRequest(req)
            
            waitForExpectations(timeout: 5, handler: nil)
            
        }
        
        func testGetAxis0Name() {
            let c = DMCPConnection(address: rmcAddress)
            
            var e = self.expectation(description: "Connect to RMC")
            c.connect { error in
                XCTAssertNil(error)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
            
            
            for i in 0...15 {
                e = self.expectation(description: "Receive response from RMC")
                
                c.getAxisName(i) { error, axisName in
                    XCTAssertNil(error)
                    print(axisName!)
                    e.fulfill()
                }

                waitForExpectations(timeout: 5, handler: nil)
            }
        }
        
        func testGetControllerName() {
            let c = DMCPConnection(address: rmcAddress)
            var e = self.expectation(description: "Connect to RMC")
            
            c.connect { error in
                XCTAssertNil(error)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
            
            e = self.expectation(description: "Get controller name from RMC")
            
            c.getControllerName { error, string in
                XCTAssertNil(error)
                print(string!)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testSendMoveAbsolute() {
            let c = DMCPConnection(address: rmcAddress)
            var e = self.expectation(description: "Connect to RMC")
            
            c.connect { error in
                XCTAssertNil(error)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
            
            e = self.expectation(description: "Receive response from RMC")
            
            let command = MoveAbsolute(to: 12, atSpeed: 1.0, withAccelRate: 3.4, withDecelRate: 1.2)
            c.issueCommand(command, toAxis: 2) { error in
                XCTAssertNil(error)
                e.fulfill()
            }

            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
//        func testTranslateAddressWithAxisName() {
//            let c = DMCPConnection(address: rmcAddress)
//            var e = self.expectation(description: "Connect to RMC")
//            
//            c.connect { error in
//                XCTAssertNil(error)
//                e.fulfill()
//            }
//            
//            waitForExpectations(timeout: 5, handler: nil)
//            
//            e = self.expectation(description: "Receive response from RMC")
//            
//            // Axis 1 actual velocity
//            let address = DMCPAddress(0 + 8, 9)
//            
//            let (axisNo, displayName): (Int?, String?) = (c.rmcType?.toRMCClass().displayName(for: address))!
//            if displayName != nil && axisNo != nil {
//                c.getAxisName(axisNo!) { error, name in
//                    print("\(name!) \(displayName!)")
//                    e.fulfill()
//                }
//            }
//
//            
//            waitForExpectations(timeout: 5, handler: nil)
//        }
        
        func testGetAxisType() {
            let c = DMCPConnection(address: rmcAddress)
            var e = self.expectation(description: "Connect to RMC")
            
            c.connect { error in
                XCTAssertNil(error)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
            
            e = self.expectation(description: "Receive response from RMC")
            
        
            c.getAxisType(forAxis: 0) { error, axisInfo in
                XCTAssertNil(error)
                print(axisInfo!)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testGetAxisStatusRegisterName() {
            let c = DMCPConnection(address: rmcAddress)
            var e = self.expectation(description: "Connect to RMC")
            
            c.connect { error in
                XCTAssertNil(error)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
            
            e = self.expectation(description: "Receive response from RMC")
            
            let address = DMCPAddress(0 + 8, 33)
            c.getAddressLabel(for: address) { error, label in
                XCTAssertNil(error)
                print(error, label)
                e.fulfill()
            }
            
            waitForExpectations(timeout: 5, handler: nil)
        }
        
        func testSeveralReadRequests() {
            let c = DMCPConnection(address: rmcAddress)
            
            c.connect { error in XCTAssert(error == nil) }
            
            for _ in 0..<100 {
                let expectation = self.expectation(description: "Receive response from RMC")
                
                let req = DMCPReadRequest(56, 0, readTypes: [.real, .real, .int, .int]) { error, values in
                    XCTAssert(error == nil)
                    print(values)
                    expectation.fulfill()
                }
                
                c.sendRequest(req)
            }
            
            waitForExpectations(timeout: 9999, handler: nil)
            
        }
    }
